/*
 *************************************************************************
 *
 * serdisp_colour.h
 * colour and colour space specific routines
 *
 *************************************************************************
 *
 * copyright (C) 2003-2006  wolfgang astleitner
 * email     mrwastl@users.sourceforge.net
 *
 *************************************************************************
 * This program is free software; you can redistribute it and/or modify   
 * it under the terms of the GNU General Public License as published by   
 * the Free Software Foundation; either version 2 of the License, or (at  
 * your option) any later version.                                        
 *                                                                        
 * This program is distributed in the hope that it will be useful, but    
 * WITHOUT ANY WARRANTY; without even the implied warranty of             
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      
 * General Public License for more details.                               
 *                                                                        
 * You should have received a copy of the GNU General Public License      
 * along with this program; if not, write to the Free Software            
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA              
 * 02111-1307, USA.  Or, point your browser to                            
 * http://www.gnu.org/copyleft/gpl.html                                   
 *************************************************************************
 */

#ifndef SERDISP_COLOUR_H
#define SERDISP_COLOUR_H

#include "serdisp_control.h"

/* define 'byte' if not available yet */
#ifndef byte
  #define byte unsigned char
#endif


/* colour spaces */

/* no flag set: auto-detect */
#define SD_CS_AUTO                0x00000000

/* bits 0-3 reserved for flags */

/* bits 4-7: greyscale colour spaces */
#define SD_CS_GREYSCALE           0x00000010

/* bits 8-15 reserved for indexed colours */

/* bits 16-27: packed colour spaces */
#define SD_CS_RGB332              0x00010000
#define SD_CS_RGB333              0x00020000
#define SD_CS_RGB444              0x00040000

/* bits 28-31: true colour spaces */
#define SD_CS_TRUECOLOUR          0x40000000
#define SD_CS_ATRUECOLOUR         0x80000000

/* mask of all indexed colour spaces */
#define SD_CS_INDEXED_SPACE       SD_CS_GREYSCALE

#define SD_CS_ISGREY(_dd)         ( ( ((_dd)->colour_spaces) & 0x000000F0L) ? 1 : 0 )
#define SD_CS_ISMONOCHROME(_dd)   ( ( ((_dd)->depth) == 1 ) ? 1 : 0 )

/* true colour or packed colour space */
#define SD_CS_ISDIRECTCOLOUR(_dd) ( ( (((_dd)->colour_spaces) & 0xFFFF0000L) && (((_dd)->depth) >= 8) ) ? 1 : 0 )

/* true colour space */
#define SD_CS_ISTRUECOLOUR(_dd)   ( ( (((_dd)->colour_spaces) & 0xF0000000L) && (((_dd)->depth) >= 24) ) ? 1 : 0 )

/* some colour defines */
#define SD_COL_BLACK              0xFF000000
#define SD_COL_WHITE              0xFFFFFFFF
#define SD_COL_RED                0xFFFF0000
#define SD_COL_GREEN              0xFF00FF00
#define SD_COL_BLUE               0xFF0000FF


#ifdef __cplusplus
extern "C" {
#endif
  void     serdisp_setcolour           (serdisp_t* dd, int x, int y, long colour);
  void     serdisp_setgrey             (serdisp_t* dd, int x, int y, byte grey);

  long     serdisp_getcolour           (serdisp_t* dd, int x, int y);
  byte     serdisp_getgrey             (serdisp_t* dd, int x, int y);

  long     serdisp_transcolour         (serdisp_t* dd, long colour);
  long     serdisp_transgrey           (serdisp_t* dd, byte greyvalue);

  long     serdisp_lookupcolour        (serdisp_t* dd, long colour);
  byte     serdisp_lookupgrey          (serdisp_t* dd, long colour);

  void     serdisp_setcoltabentry      (serdisp_t* dd, int idx, long colour);
  long     serdisp_getcoltabentry      (serdisp_t* dd, int idx);



  /* colour space specific routines */
  long     serdisp_transcolour_bw      (serdisp_t* dd, long colour);
  long     serdisp_transgrey_bw        (serdisp_t* dd, byte greyvalue);
  long     serdisp_lookupcolour_bw     (serdisp_t* dd, long colour);
  byte     serdisp_lookupgrey_bw       (serdisp_t* dd, long colour);
  
  long     serdisp_transcolour_grey2_4 (serdisp_t* dd, long colour);
  long     serdisp_transgrey_grey2_4   (serdisp_t* dd, byte greyvalue);
  long     serdisp_lookupcolour_grey2_4(serdisp_t* dd, long colour);
  byte     serdisp_lookupgrey_grey2_4  (serdisp_t* dd, long colour);
  
  long     serdisp_transcolour_grey8   (serdisp_t* dd, long colour);
  long     serdisp_transgrey_grey8     (serdisp_t* dd, byte greyvalue);
  long     serdisp_lookupcolour_grey8  (serdisp_t* dd, long colour);
  byte     serdisp_lookupgrey_grey8    (serdisp_t* dd, long colour);
  
  long     serdisp_transcolour_rgb332  (serdisp_t* dd, long colour);
  long     serdisp_transgrey_rgb332    (serdisp_t* dd, byte greyvalue);
  long     serdisp_lookupcolour_rgb332 (serdisp_t* dd, long colour);
  byte     serdisp_lookupgrey_rgb332   (serdisp_t* dd, long colour);
  
  long     serdisp_transcolour_rgb444  (serdisp_t* dd, long colour);
  long     serdisp_transgrey_rgb444    (serdisp_t* dd, byte greyvalue);
  long     serdisp_lookupcolour_rgb444 (serdisp_t* dd, long colour);
  byte     serdisp_lookupgrey_rgb444   (serdisp_t* dd, long colour);
  
#ifdef __cplusplus
    }
#endif


/* some macros to make life easier */
#define serdisp_pack2ARGB(_a,_r,_g,_b)        ( (((long)(_a))<<24) | (((long)(_r))<<16) | (((long)(_g))<<8) | (long)(_b) )

#define serdisp_ARGB2GREY(_col)               ((byte)((((0x00FF0000 & (_col)) >> 16) * 77 + ((0x0000FF00 & (_col)) >> 8) * 150 + (0x000000FF & (_col)) * 28) / 255))
#define serdisp_GREY2ARGB(_grey)              ( 0xFF000000 | (((long)(_grey))<<16) | (((long)(_grey))<<8) | ((long)(_grey)) )

#define serdisp_COLCROSSTOTAL(_col)           (((_col) & 0xFF) + (((_col) & 0xFF00) >> 8) + (((_col) & 0xFF0000) >> 16))

#endif /* SERDISP_COLOUR_H */
